<?php
/* ZNCServer.class.php - ZNC Server Class - ZNCAdmin
 * Copyright (C) 2011-2012  Philipp Kreil (pk910)
 * 
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation, either version 3 of the License, or
 * (at your option) any later version.
 * 
 * This program is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 * 
 * You should have received a copy of the GNU General Public License 
 * along with this program. If not, see <http://www.gnu.org/licenses/>. 
 */

class ZNCServer {
	private $host, $port;
	private $connector;
	
	public function ZNCServer($host, $port) {
		$this->host = $host;
		$this->port = $port;
		$this->connector = new HTTPConnector();
	}
	
	public function login($user, $pass) {
		$post = array(
			"user" => $user,
			"pass" => $pass,
			"submitted" => "1"
		);
		$this->connector->post("http://".$this->host.":".$this->port."/login", $post);
		$http = $this->connector->get("http://".$this->host.":".$this->port."/?cookie_check=true");
		$logged_in = !preg_match("/errorbar/i", $http);
		
		return $logged_in;
	}
	
	public function getUserList() {
		$html = $this->connector->get("http://".$this->host.":".$this->port."/mods/webadmin/listusers");
		$exp = explode('<div class="toptable">', $html);
		$exp = explode('</div>', $exp[1]);
		$exp = explode('<tbody>', $exp[0]);
		$exp = explode('</tbody>', $exp[1]);
		$list = explode('<tr class=', $exp[0]);
		$userlist = array();
		for($i = 1; $i < count($list); $i++) {
			$userdata = array();
			$exp = explode('<td>', $list[$i]);
			$exp2 = explode('</td>', $exp[2]);
			$userdata['user'] = $exp2[0];
			$exp2 = explode('</td>', $exp[3]);
			$userdata['clients'] = $exp2[0];
			$exp2 = explode('</td>', $exp[4]);
			$userdata['server'] = $exp2[0];
			$exp2 = explode('</td>', $exp[5]);
			$userdata['nick'] = $exp2[0];
			$userlist[] = $userdata;
		}
		return $userlist;
	}
    
    public function getSeenList() {
		$html = $this->connector->get("http://".$this->host.":".$this->port."/mods/lastseen/");
		$exp = explode('<div class="toptable">', $html);
		$exp = explode('</div>', $exp[1]);
		$exp = explode('<tbody>', $exp[0]);
		$exp = explode('</tbody>', $exp[1]);
		$list = explode('<tr class=', $exp[0]);
		$seenlist = array();
		for($i = 1; $i < count($list); $i++) {
			$userdata = array();
			$exp = explode('<td>', $list[$i]);
            
			$exp2 = explode('</td>', $exp[1]);
			$userdata['user'] = $exp2[0];
			$exp2 = explode('</td>', $exp[2]);
			$userdata['seen'] = $exp2[0];
            $userdata['seen_unix'] = strtotime($exp2[0]);
			$exp2 = explode('</td>', $exp[3]);
			$userdata['info'] = $exp2[0];
			$seenlist[] = $userdata;
		}
		return $seenlist;
	}
    
    /*
    $settings array elements:
        nick
        altnick
        ident
    */
    public function addZNC($username, $password, $settings, $servers, $modules, $others) {
        $html = $this->connector->get("http://".$this->host.":".$this->port."/mods/webadmin/adduser");
		$exp = explode('name="_CSRF_Check" value="', $html);
        $exp = explode('"', $exp[1]);
        $csrf = $exp[0];
        $post = array();
        $post['_CSRF_Check'] = $csrf;
        $post['submitted'] = '1';
        $post['doconnect'] = '1';
        $post['user'] = $username;
        $post['password'] = $password;
        $post['password2'] = $password;
        $post['nick'] = $settings['nick'];
        $post['altnick'] = $settings['altnick'];
        $post['ident'] = $settings['ident'];
        $post['statusprefix'] = '*';
        $post['servers'] = implode("\n", $servers);
        $post['loadmod'] = array();
        foreach($modules as $name => $args) {
            $post['loadmod'][] = $name;
            if($args != "")
                $post['modargs_'.$name] = $args;
        }
        foreach($others as $name => $value) {
            $post[$name] = $value;
        }
        $html = $this->connector->post("http://".$this->host.":".$this->port."/mods/webadmin/adduser", $post);
        return !preg_match("/Invalid Submission/i", $html);
    }
	
    public function delZNC($username) {
        $html = $this->connector->get("http://".$this->host.":".$this->port."/mods/webadmin/deluser?user=".$username);
		$exp = explode('name="_CSRF_Check" value="', $html);
        $exp = explode('"', $exp[1]);
        $csrf = $exp[0];
        $post = array();
        $post['_CSRF_Check'] = $csrf;
        $post['submitted'] = '1';
        $post['user'] = $username;
        $this->connector->post("http://".$this->host.":".$this->port."/mods/webadmin/deluser", $post);
    }
    
    public function editZNC($username, $password = NULL, $new_servers = NULL, $add_modules = NULL, $del_modules = NULL, $others = NULL) {
        $html = $this->connector->get("http://".$this->host.":".$this->port."/mods/webadmin/edituser?user=".$username);
        if(preg_match("/No such username/i", $html)) return false;
        $post = array();
        $post['loadmod'] = array();
        preg_match_all("<input ([^\>]*)>", $html, $matches);
        foreach($matches[0] as $input) {
            $fields = array(0 => "");
            $esc = false; $str = false;
            $fieldid = 0;
            for($i = 0; $i < strlen($input); $i++) {
                if($esc) {
                    $esc = false;
                    $fields[$fieldid] .= $input[$i];
                    continue;
                }
                if($input[$i] == "\\") {
                    $esc = true;
                }
                if($input[$i] == "\"") {
                    $str = !$str;
                    continue;
                }
                if($input[$i] == " " && !$str) {
                    $fields[++$fieldid] = "";
                    continue;
                }
                $fields[$fieldid] .= $input[$i];
            }
            $cfields = array();
            foreach($fields as $field) {
                $fieldexp = explode("=", $field, 2);
                if(count($fieldexp) != 2) continue;
                $cfields[$fieldexp[0]] = $fieldexp[1];
            }
            $name = $cfields['name'];
            switch($cfields['type']) {
                case "text":
                case "hidden":
                    $value = $cfields['value'];
                    break;
                case "checkbox":
                    if($cfields['checked']) {
                        $value = ($cfields['value'] ? $cfields['value'] : "1");
                    } else 
                        $value = null;
                    break;
                default:
                    $value = null;
            }
            if($name && $value) {
                if(array_key_exists($name, $post)) {
                    if(!is_array($post[$name])) {
                        $cvalue = $post[$name];
                        $post[$name] = array();
                        $post[$name][] = $cvalue;
                    }
                    $post[$name][] = $value;
                } else
                    $post[$name] = $value;
            }
        }
        //textareas
        $textareas = explode("<textarea ", $html);
        for($i = 1; $i < count($textareas); $i++) {
            $exp = explode(">", $textareas[$i]);
            $name = explode("name=\"", $exp[0]);
            $name = explode("\"", $name[1]);
            $name = $name[0];
            $content = explode("</textarea", $exp[1]);
            $content = $content[0];
            $post[$name] = $content;
        }
        if($password) {
            $post['password'] = $password;
            $post['password2'] = $password;
        }
        if($new_servers) {
            $post['servers'] = implode("\n", $new_servers);
        }
        foreach($add_modules as $name => $args) {
            $post['loadmod'][] = $name;
            if($args != "")
                $post['modargs_'.$name] = $args;
        }
        foreach($del_modules as $name => $args) {
            foreach($post['loadmod'] as $index => $mod) {
                if(strtolower($mod) == strtolower($name))
                    unset($post['loadmod'][$index]);
            }
        }
        foreach($others as $name => $value) {
            $post[$name] = $value;
        }
        $html = $this->connector->post("http://".$this->host.":".$this->port."/mods/webadmin/edituser?user=".$username, $post);
        return !preg_match("/Invalid Submission/i", $html);
    }
    
    public function simulZNC($username, $raw, $server = true) {
        $html = $this->connector->get("http://".$this->host.":".$this->port."/mods/send_raw/");
        if(preg_match("/Not Found/i", $html)) return false;
		$exp = explode('name="_CSRF_Check" value="', $html);
        $exp = explode('"', $exp[1]);
        $csrf = $exp[0];
        $post = array();
        $post['_CSRF_Check'] = $csrf;
        $post['submitted'] = '1';
        $post['user'] = $username;
        $post['send_to'] = ($server ? "server" : "client");
        $post['line'] = $raw;
        $this->connector->post("http://".$this->host.":".$this->port."/mods/send_raw/", $post);
    }
    
    public function addChan($username, $channel, $key = NULL, $buffer = NULL) {
        $html = $this->connector->get("http://".$this->host.":".$this->port."/mods/webadmin/addchan?user=".$username);
        if(preg_match("/No such username/i", $html)) return false;
		$exp = explode('name="_CSRF_Check" value="', $html);
        $exp = explode('"', $exp[1]);
        $csrf = $exp[0];
        $post = array();
        $post['_CSRF_Check'] = $csrf;
        $post['submitted'] = '1';
        $post['user'] = $username;
        $post['name'] = $channel;
        $exp = explode('name="key" value="', $html);
        $exp = explode('"', $exp[1]);
        $default_key = $exp[0];
        $post['key'] = ($key ? $key : $default_key);
        $exp = explode('name="buffercount" value="', $html);
        $exp = explode('"', $exp[1]);
        $default_buffer = $exp[0];
        $post['buffercount'] = ($buffer ? $buffer : $default_buffer);
        $post['save'] = 'true';
        $this->connector->post("http://".$this->host.":".$this->port."/mods/webadmin/addchan?user=".$username, $post);
    }
    
}

?>